@@ -499,8 +499,9 @@ class AdsorbateGroupSubstitute(Mutation):
 
     def get_new_individual(self, parents):
         f = parents[0]
-
+        print('f: {}'.format(f))
         indi = self.substitute(f)
+        print('indi: {}'.format(indi))
         indi = self.initialize_individual(f, indi) 
         if 'groups' in f.info['data']:
             indi.info['data']['groups'] = f.info['data']['groups']
